@@ -3,11 +3,11 @@ use crate::select::SelectColumns;
 use crate::util;
 use crate::CliResult;
 use cached::proc_macro::{cached, io_cached};
-use cached::RedisCache;
-use cached::Return;
+use cached::{RedisCache, Return};
+use dynfmt::{Format, SimpleCurlyFormat};
 use indicatif::{ProgressBar, ProgressDrawTarget};
 use log::{debug, error};
-use once_cell::sync::Lazy;
+use once_cell::sync::{Lazy, OnceCell};
 use serde::Deserialize;
 use thiserror::Error;
 
@@ -15,30 +15,80 @@ static USAGE: &str = "
 This command fetches data from a web API for every row in the URL column, 
 and optionally stores them in a new column.
 
-Fetch is integrated with `jql` to directly parse out values from API JSON response.
+Fetch is integrated with `jql` to directly parse out values from an API JSON response.
 
-URL column can either be a fully qualified URL path, or if not, can be used with
-the --url-template option to create one.
+The URL column needs to be a fully qualified URL path. Alternatively, you can dynamically
+construct URLs for each CSV record with the --url-template option (see Examples below).
 
 To use a proxy, please set env vars HTTP_PROXY and HTTPS_PROXY
 (e.g. export HTTPS_PROXY=socks5://127.0.0.1:1086).
 
-Set the --redis flag to use Redis. By default, it will connect to a local Redis instance 
-at redis://127.0.0.1:6379, with a cache expiry TTL of 2,419,200 seconds (28 days), and
-cache hits NOT refreshing the TTL of cached values.
+To use Redis for response caching, set the --redis flag. By default, it will connect to
+a local Redis instance at redis://127.0.0.1:6379, with a cache expiry Time-to-Live (TTL)
+of 2,419,200 seconds (28 days), and cache hits NOT refreshing the TTL of cached values.
 
 Set the env vars QSV_REDIS_CONNECTION_STRING, QSV_REDIS_TTL_SECONDS and 
-QSV_REDIS_TTL_REFRESH to change default settings.
+QSV_REDIS_TTL_REFRESH to change default Redis settings.
+
+EXAMPLES USING THE COLUMN ARGUMENT:
+
+data.csv
+  URL
+  http://api.zippopotam.us/us/90210
+  http://api.zippopotam.us/us/94105
+  https://api.zippopotam.us/us/92802
+
+Given the data.csv above, fetch the JSON response.
+
+  $ qsv fetch URL data.csv 
+
+Note the output will just be a JSONL file - with a minified JSON response per line,
+not a CSV file.
+
+Now, if we want to generate a CSV file with the parsed City and State, we use the 
+new-column and jql options.
+
+  $ qsv fetch URL --new-column CityState \
+    --jql '\"\"\"places\"\"\"[0].\"\"\"place name\"\"\",\"\"\"places\"\"\"[0].\"\"\"state abbreviation\"\"\"' \
+    data.csv > datatest.csv
+
+data_with_CityState.csv
+  URL, CityState,
+  http://api.zippopotam.us/us/90210, \"Beverly Hills, CA\"
+  http://api.zippopotam.us/us/94105, \"San Francisco, CA\"
+  https://api.zippopotam.us/us/92802, \"Anaheim, CA\"
+
+
+EXAMPLES USING THE --URL-TEMPLATE OPTION:
+
+Geocode addresses in addr_data.csv, pass the latitude and longitude fields and store
+the response in a new column called response into enriched_addr_data.csv. Since we're
+using --url-template, let's just specify 1 for the column parameter as its required.
+
+  $ qsv fetch 1 --url-template \"https://geocode.test/api/lookup.json?lat={latitude}&long={longitude}\" \
+       addr_data.csv -c response > enriched_addr_data.csv
+
+Geocode addresses in addr_data.csv, pass the \"street address\" and \"zip-code\" fields
+and use jql to parse CityState from the JSON response into a new column in enriched.csv.
+Note how field name non-alphanumeric characters in the url-template were replace with _.
+
+  $ qsv fetch 1 --url-template \"https://geocode.test/api/addr.json?addr={street_address}&zip={zip_code}\" \
+       --jql '\"\"\"places\"\"\"[0].\"\"\"place name\"\"\",\"\"\"places\"\"\"[0].\"\"\"state abbreviation\"\"\"' \
+       addr_data.csv -c CityState > enriched.csv
 
 Usage:
     qsv fetch [options] [--http-header <k:v>...] [<column>] [<input>]
 
 Fetch options:
-    --url-template <template>  URL template to use. The character '^'
-                               will be replaced by <column> value, but
-                               sanitized for shell safety.
+    --url-template <template>  URL template to use. Use column names enclosed with
+                               curly braces to insert the CSV data for a record.
+                               When using this option, the column argument is ignored
+                               but still required. Just use the value '1' for column
+                               when using url-template.
     -c, --new-column <name>    Put the fetched values in a new column instead.
     --jql <selector>           Apply jql selector to API returned JSON value.
+    --pretty                   Prettify JSON responses. Otherwise, they're minified.
+                               If the response is not in JSON format, it's passed through.
     --rate-limit <qps>         Rate Limit in Queries Per Second. [default: 10]
     --http-header <key:value>  Pass custom header(s) to the server.
     --store-error              On error, store error code/message instead of blank value.
@@ -62,6 +112,7 @@ struct Args {
     flag_url_template: Option<String>,
     flag_new_column: Option<String>,
     flag_jql: Option<String>,
+    flag_pretty: bool,
     flag_rate_limit: Option<u32>,
     flag_http_header: Vec<String>,
     flag_store_error: bool,
@@ -78,13 +129,6 @@ struct Args {
 static DEFAULT_REDIS_CONN_STR: &str = "redis://127.0.0.1:6379";
 static DEFAULT_REDIS_TTL_SECONDS: u64 = 60 * 60 * 24 * 28; // 28 days in seconds
 
-static DEFAULT_USER_AGENT: &str = concat!(
-    env!("CARGO_PKG_NAME"),
-    "/",
-    env!("CARGO_PKG_VERSION"),
-    " (https://github.com/jqnatividad/qsv)",
-);
-
 struct RedisConfig {
     conn_str: String,
     ttl_secs: u64,
@@ -106,6 +150,8 @@ impl RedisConfig {
 
 static REDISCONFIG: Lazy<RedisConfig> = Lazy::new(RedisConfig::load);
 
+static JQL_GROUPS: once_cell::sync::OnceCell<Vec<jql::Group>> = OnceCell::new();
+
 #[derive(Error, Debug, PartialEq, Clone)]
 enum FetchRedisError {
     #[error("error with redis cache `{0}`")]
@@ -124,23 +170,29 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     let mut wtr = Config::new(&args.flag_output).writer()?;
 
     let mut headers = rdr.byte_headers()?.clone();
+
     let sel = rconfig.selection(&headers)?;
     let column_index = *sel.iter().next().unwrap();
 
-    assert!(
-        sel.len() == 1,
-        "Only one single URL column may be selected."
-    );
+    if sel.len() != 1 {
+        return fail!("Only one single URL column may be selected.");
+    }
+
+    if args.flag_no_headers && args.flag_url_template.is_some() {
+        return fail!("--url-template option requires headers.");
+    }
+
+    let str_headers = rdr.headers()?.clone();
+    let safe_headers = util::safe_header_names(str_headers, false);
 
     use std::num::NonZeroU32;
     // default rate limit is actually set via docopt, so below init is just to satisfy compiler
     let mut rate_limit: NonZeroU32 = NonZeroU32::new(10).unwrap();
     if let Some(qps) = args.flag_rate_limit {
-        assert!(
-            // on my laptop, no more sleep traces with qps > 24, so use round number of 20 as single-thread qps limit
-            qps <= 20 && qps > 0,
-            "Rate Limit should be between 1 to 20 queries per second."
-        );
+        // on my laptop, no more sleep traces with qps > 24, so use round number of 20 as single-thread qps limit
+        if !(qps <= 20 && qps > 0) {
+            return fail!("Rate Limit should be between 1 to 20 queries per second.");
+        }
         rate_limit = NonZeroU32::new(qps).unwrap();
     }
 
@@ -168,9 +220,11 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
 
     use reqwest::blocking::Client;
     let client = Client::builder()
-        .user_agent(DEFAULT_USER_AGENT)
+        .user_agent(util::DEFAULT_USER_AGENT)
         .default_headers(http_headers)
         .cookie_store(args.flag_cookies)
+        .brotli(true)
+        .gzip(true)
         .build()
         .unwrap();
 
@@ -201,24 +255,31 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
     #[allow(unused_assignments)]
     let mut record = csv::ByteRecord::new();
     #[allow(unused_assignments)]
-    let mut url = String::new();
+    let mut url = String::default();
     let mut redis_cache_hits: u64 = 0;
+    #[allow(unused_assignments)]
+    let mut final_value = String::default();
+
     while rdr.read_byte_record(&mut record)? {
         if !args.flag_quiet {
             progress.inc(1);
         }
 
-        #[allow(unused_assignments)]
-        let mut final_value = String::default();
-
         if let Ok(s) = std::str::from_utf8(&record[column_index]) {
-            match args.flag_url_template {
-                Some(ref url_template) => {
-                    url = url_template.replace('^', s.trim());
-                }
-                _ => url = s.trim().to_string(),
-            }
+            if let Some(ref url_template) = args.flag_url_template {
+                let mut record_map = std::collections::BTreeMap::new();
 
+                for (i, field) in record.iter().enumerate() {
+                    let str_value =
+                        unsafe { std::str::from_utf8_unchecked(field).trim().to_string() };
+                    record_map.insert(safe_headers[i].to_owned(), str_value);
+                }
+                if let Ok(formatted_url) = SimpleCurlyFormat.format(url_template, record_map) {
+                    url = formatted_url.to_string();
+                }
+            } else {
+                url = s.trim().to_string()
+            }
             debug!("Fetching URL: {url:?}");
 
             if args.flag_redis {
@@ -228,6 +289,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     &limiter,
                     &args.flag_jql,
                     args.flag_store_error,
+                    args.flag_pretty,
                 )
                 .unwrap();
                 final_value = intermediate_value.to_string();
@@ -241,6 +303,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                     &limiter,
                     &args.flag_jql,
                     args.flag_store_error,
+                    args.flag_pretty,
                 );
             }
         } else {
@@ -285,8 +348,16 @@ fn get_cached_response(
     limiter: &governor::RateLimiter<NotKeyed, InMemoryState, DefaultClock, NoOpMiddleware>,
     flag_jql: &Option<String>,
     flag_store_error: bool,
+    flag_pretty: bool,
 ) -> String {
-    get_response(url, client, limiter, flag_jql, flag_store_error)
+    get_response(
+        url,
+        client,
+        limiter,
+        flag_jql,
+        flag_store_error,
+        flag_pretty,
+    )
 }
 
 #[io_cached(
@@ -310,6 +381,7 @@ fn get_redis_response(
     limiter: &governor::RateLimiter<NotKeyed, InMemoryState, DefaultClock, NoOpMiddleware>,
     flag_jql: &Option<String>,
     flag_store_error: bool,
+    flag_pretty: bool,
 ) -> Result<cached::Return<String>, FetchRedisError> {
     Ok(Return::new(get_response(
         url,
@@ -317,6 +389,7 @@ fn get_redis_response(
         limiter,
         flag_jql,
         flag_store_error,
+        flag_pretty,
     )))
 }
 
@@ -326,6 +399,7 @@ fn get_response(
     limiter: &governor::RateLimiter<NotKeyed, InMemoryState, DefaultClock, NoOpMiddleware>,
     flag_jql: &Option<String>,
     flag_store_error: bool,
+    flag_pretty: bool,
 ) -> String {
     // wait until RateLimiter gives Okay
     while limiter.check().is_err() {
@@ -374,7 +448,10 @@ fn get_response(
     } else {
         // apply JQL selector if provided
         if let Some(selectors) = flag_jql {
-            match apply_jql(&api_value, selectors) {
+            // instead of repeatedly parsing the jql selector,
+            // we compile it only once and cache it for performance using once_cell
+            let jql_groups = JQL_GROUPS.get_or_init(|| jql::selectors_parser(selectors).unwrap());
+            match apply_jql(&api_value, jql_groups) {
                 Ok(s) => {
                     final_value = s;
                 }
@@ -390,6 +467,14 @@ fn get_response(
                     }
                 }
             }
+        } else if flag_pretty {
+            if let Ok(pretty_json) = jsonxf::pretty_print(&api_value) {
+                final_value = pretty_json;
+            } else {
+                final_value = api_value;
+            }
+        } else if let Ok(minimized_json) = jsonxf::minimize(&api_value) {
+            final_value = minimized_json;
         } else {
             final_value = api_value;
         }
@@ -400,12 +485,12 @@ fn get_response(
     final_value
 }
 
-use jql::walker;
+use jql::groups_walker;
 use serde_json::{Deserializer, Value};
 
 use anyhow::{anyhow, Result};
 
-fn apply_jql(json: &str, selectors: &str) -> Result<String> {
+fn apply_jql(json: &str, groups: &[jql::Group]) -> Result<String> {
     // check if api returned valid JSON before applying JQL selector
     if let Err(error) = serde_json::from_str::<Value>(json) {
         return Err(anyhow!("Invalid json: {error:?}"));
@@ -419,7 +504,7 @@ fn apply_jql(json: &str, selectors: &str) -> Result<String> {
             Ok(valid_json) => {
                 // Walk through the JSON content with the provided selectors as
                 // input.
-                match walker(&valid_json, selectors) {
+                match groups_walker(&valid_json, groups) {
                     Ok(selection) => {
                         fn get_value_string(v: &Value) -> String {
                             if v.is_null() {
@@ -486,7 +571,8 @@ fn test_apply_jql_invalid_json() {
         r#"<!doctype html><html lang="en"><meta charset=utf-8><title>shortest html5</title>"#;
     let selectors = r#"."places"[0]."place name""#;
 
-    let value: String = apply_jql(json, selectors).unwrap_err().to_string();
+    let jql_groups = jql::selectors_parser(selectors).unwrap();
+    let value: String = apply_jql(json, &jql_groups).unwrap_err().to_string();
 
     assert_eq!(
         "Invalid json: Error(\"expected value\", line: 1, column: 1)",
@@ -499,7 +585,8 @@ fn test_apply_jql_invalid_selector() {
     let json = r#"{"post code": "90210", "country": "United States", "country abbreviation": "US", "places": [{"place name": "Beverly Hills", "longitude": "-118.4065", "state": "California", "state abbreviation": "CA", "latitude": "34.0901"}]}"#;
     let selectors = r#"."place"[0]."place name""#;
 
-    let value = apply_jql(json, selectors).unwrap_err().to_string();
+    let jql_groups = jql::selectors_parser(selectors).unwrap();
+    let value = apply_jql(json, &jql_groups).unwrap_err().to_string();
 
     assert_eq!("Node \"place\" not found on the parent element", value);
 }
@@ -509,7 +596,8 @@ fn test_apply_jql_string() {
     let json = r#"{"post code": "90210", "country": "United States", "country abbreviation": "US", "places": [{"place name": "Beverly Hills", "longitude": "-118.4065", "state": "California", "state abbreviation": "CA", "latitude": "34.0901"}]}"#;
     let selectors = r#"."places"[0]."place name""#;
 
-    let value: String = apply_jql(json, selectors).unwrap();
+    let jql_groups = jql::selectors_parser(selectors).unwrap();
+    let value: String = apply_jql(json, &jql_groups).unwrap();
 
     assert_eq!("Beverly Hills", value);
 }
@@ -519,7 +607,8 @@ fn test_apply_jql_number() {
     let json = r#"{"post code": "90210", "country": "United States", "country abbreviation": "US", "places": [{"place name": "Beverly Hills", "longitude": -118.4065, "state": "California", "state abbreviation": "CA", "latitude": 34.0901}]}"#;
     let selectors = r#"."places"[0]."longitude""#;
 
-    let value: String = apply_jql(json, selectors).unwrap();
+    let jql_groups = jql::selectors_parser(selectors).unwrap();
+    let value: String = apply_jql(json, &jql_groups).unwrap();
 
     assert_eq!("-118.4065", value);
 }
@@ -529,9 +618,10 @@ fn test_apply_jql_bool() {
     let json = r#"{"post code": "90210", "country": "United States", "country abbreviation": "US", "places": [{"place name": "Beverly Hills", "longitude": -118.4065, "state": "California", "state abbreviation": "CA", "latitude": 34.0901, "expensive": true}]}"#;
     let selectors = r#"."places"[0]."expensive""#;
 
-    let value: String = apply_jql(json, selectors).unwrap();
+    let jql_groups = jql::selectors_parser(selectors).unwrap();
+    let value: String = apply_jql(json, &jql_groups).unwrap();
 
-    assert_eq!(true.to_string(), value);
+    assert_eq!("true", value);
 }
 
 #[test]
@@ -539,9 +629,10 @@ fn test_apply_jql_null() {
     let json = r#"{"post code": "90210", "country": "United States", "country abbreviation": "US", "places": [{"place name": "Beverly Hills", "longitude": -118.4065, "state": "California", "state abbreviation": "CA", "latitude": 34.0901, "university":null}]}"#;
     let selectors = r#"."places"[0]."university""#;
 
-    let value: String = apply_jql(json, selectors).unwrap();
+    let jql_groups = jql::selectors_parser(selectors).unwrap();
+    let value: String = apply_jql(json, &jql_groups).unwrap();
 
-    assert_eq!("null".to_string(), value);
+    assert_eq!("null", value);
 }
 
 #[test]
@@ -549,7 +640,24 @@ fn test_apply_jql_array() {
     let json = r#"{"post code": "90210", "country": "United States", "country abbreviation": "US", "places": [{"place name": "Beverly Hills", "longitude": -118.4065, "state": "California", "state abbreviation": "CA", "latitude": 34.0901}]}"#;
     let selectors = r#"."places"[0]."longitude",."places"[0]."latitude""#;
 
-    let value: String = apply_jql(json, selectors).unwrap();
+    let jql_groups = jql::selectors_parser(selectors).unwrap();
+    let value: String = apply_jql(json, &jql_groups).unwrap();
 
-    assert_eq!("-118.4065, 34.0901".to_string(), value);
+    assert_eq!("-118.4065, 34.0901", value);
+}
+
+#[test]
+fn test_root_out_of_bounds() {
+    // test for out_of_bounds root element handling
+    // see https://github.com/yamafaktory/jql/issues/129
+    let json = r#"[{"page":1,"pages":1,"per_page":"50","total":1},[{"id":"BRA","iso2Code":"BR","name":"Brazil","region":{"id":"LCN","iso2code":"ZJ","value":"Latin America & Caribbean (all income levels)"},"adminregion":{"id":"LAC","iso2code":"XJ","value":"Latin America & Caribbean (developing only)"},"incomeLevel":{"id":"UMC","iso2code":"XT","value":"Upper middle income"},"lendingType":{"id":"IBD","iso2code":"XF","value":"IBRD"},"capitalCity":"Brasilia","longitude":"-47.9292","latitude":"-15.7801"}]]"#;
+    let selectors = r#"[2].[0]."incomeLevel"."value"'"#;
+
+    let jql_groups = jql::selectors_parser(selectors).unwrap();
+    let value = apply_jql(json, &jql_groups).unwrap_err().to_string();
+
+    assert_eq!(
+        "Index [2] is out of bound, root element has a length of 2",
+        value
+    );
 }
