@@ -1,4 +1,4 @@
-#[cfg(feature = "apply")]
+#[cfg(all(feature = "apply", not(feature = "lite")))]
 pub mod apply;
 pub mod behead;
 pub mod cat;
@@ -7,35 +7,38 @@ pub mod dedup;
 pub mod enumerate;
 pub mod exclude;
 pub mod explode;
+#[cfg(all(feature = "fetch", not(feature = "lite")))]
 pub mod fetch;
 pub mod fill;
 pub mod fixlengths;
 pub mod flatten;
 pub mod fmt;
-#[cfg(feature = "foreach")]
+#[cfg(all(feature = "foreach", not(feature = "lite")))]
 pub mod foreach;
 pub mod frequency;
-#[cfg(feature = "generate")]
+#[cfg(all(feature = "generate", not(feature = "lite")))]
 pub mod generate;
 pub mod headers;
 pub mod index;
 pub mod input;
 pub mod join;
 pub mod jsonl;
-#[cfg(feature = "lua")]
+#[cfg(all(feature = "lua", not(feature = "lite")))]
 pub mod lua;
 pub mod partition;
 pub mod pseudo;
-#[cfg(feature = "python")]
+#[cfg(all(feature = "python", not(feature = "lite")))]
 pub mod python;
 pub mod rename;
 pub mod replace;
 pub mod reverse;
 pub mod sample;
+pub mod schema;
 pub mod search;
 pub mod searchset;
 pub mod select;
 pub mod slice;
+pub mod sniff;
 pub mod sort;
 pub mod split;
 pub mod stats;
