@@ -183,8 +183,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         progress.set_draw_target(ProgressDrawTarget::hidden());
     }
 
-    let mut record = csv::StringRecord::new();
+    let header_vec = util::safe_header_names(headers, true);
 
+    let mut record = csv::StringRecord::new();
     while rdr.read_record(&mut record)? {
         if !args.flag_quiet {
             progress.inc(1);
@@ -193,9 +194,9 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         // Initializing locals
         let mut row_data: Vec<&str> = Vec::with_capacity(headers_len);
 
-        for (i, h) in headers.iter().take(headers_len).enumerate() {
+        for (i, key) in header_vec.iter().enumerate().take(headers_len) {
             let cell_value = record.get(i).unwrap();
-            locals.set_item(h.replace(" ", "_"), cell_value)?;
+            locals.set_item(key, cell_value)?;
             row_data.push(cell_value);
         }
 
