@@ -1,15 +1,13 @@
 use std::io;
 
-use byteorder::{ByteOrder, LittleEndian};
-use rand::rngs::StdRng;
-use rand::seq::SliceRandom;
-use rand::{self, Rng, SeedableRng};
+use rand::{self, rngs::StdRng, seq::SliceRandom, Rng, SeedableRng};
 
 use crate::config::{Config, Delimiter};
 use crate::index::Indexed;
-use crate::serde::Deserialize;
 use crate::util;
 use crate::CliResult;
+use log::debug;
+use serde::Deserialize;
 
 static USAGE: &str = "
 Randomly samples CSV data uniformly using memory proportional to the size of
@@ -80,6 +78,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
             }
         }
         _ => {
+            debug!("no index");
             if sample_size < 1.0 {
                 return fail!("Percentage sampling requires an index.");
             }
@@ -102,6 +101,7 @@ where
     R: io::Read + io::Seek,
     I: io::Read + io::Seek,
 {
+    debug!("doing sample_random_access");
     let mut all_indices = (0..idx.count()).collect::<Vec<_>>();
     let mut rng = ::rand::thread_rng();
     SliceRandom::shuffle(&mut *all_indices, &mut rng);
@@ -119,6 +119,7 @@ fn sample_reservoir<R: io::Read>(
     sample_size: u64,
     seed: Option<usize>,
 ) -> CliResult<Vec<csv::ByteRecord>> {
+    debug!("doing sample_reservoir");
     // The following algorithm has been adapted from:
     // https://en.wikipedia.org/wiki/Reservoir_sampling
     let mut reservoir = Vec::with_capacity(sample_size as usize);
@@ -130,16 +131,12 @@ fn sample_reservoir<R: io::Read>(
     // Seeding rng
     let mut rng: StdRng = match seed {
         None => StdRng::from_rng(rand::thread_rng()).unwrap(),
-        Some(seed) => {
-            let mut buf = [0u8; 32];
-            LittleEndian::write_u64(&mut buf, seed as u64);
-            SeedableRng::from_seed(buf)
-        }
+        Some(seed) => StdRng::seed_from_u64(seed as u64),
     };
 
     // Now do the sampling.
     for (i, row) in records {
-        let random = rng.gen_range(0, i + 1);
+        let random = rng.gen_range(0..i + 1);
         if random < sample_size as usize {
             reservoir[random] = row?;
         }
@@ -148,5 +145,7 @@ fn sample_reservoir<R: io::Read>(
 }
 
 fn do_random_access(sample_size: u64, total: u64) -> bool {
-    sample_size <= (total / 10)
+    let raflag = sample_size <= (total / 10) as u64;
+    debug!("sample_size: {sample_size}, total: {total}, raflag: {raflag}");
+    raflag
 }
