@@ -24,14 +24,17 @@ static USAGE: &str = "
 Computes basic statistics on CSV data.
 
 Basic statistics includes sum, min/max, min/max length, mean, stddev, variance,
-quartiles, median, mode, cardinality & nullcount. Note that some statistics are
-expensive to compute, so they must be enabled explicitly. By default, the following
-statistics are reported for *every* column in the CSV data: sum, min/max values,
-min/max length, mean, stddev & variance. The default set of statistics corresponds to
-statistics that can be computed efficiently on a stream of data (i.e., constant memory).
+quartiles, median, modes, cardinality & nullcount. Note that some statistics are
+expensive to compute and requires loading the entire file into memory,
+so they must be enabled explicitly. 
+
+By default, the following statistics are reported for *every* column in the CSV data:
+sum, min/max values, min/max length, mean, stddev & variance. The default set of statistics 
+corresponds to statistics that can be computed efficiently on a stream of data (i.e., constant memory).
 
 The data type of each column is also inferred (Unknown, NULL, Integer, String,
-Float, Date and DateTime). The date formats recognized can be found at
+Float, Date and DateTime). Note that the Date and DateTime data types are only inferred with
+the --dates option as its an expensive operation. The date formats recognized can be found at
 https://docs.rs/dateparser/0.1.6/dateparser/#accepted-date-formats.
 
 Computing statistics on a large file can be made much faster if you create
@@ -172,7 +175,7 @@ impl Args {
         let mut records: Vec<_> = repeat(csv::StringRecord::new()).take(stats.len()).collect();
         let pool = ThreadPool::new(self.njobs());
         let mut results = vec![];
-        for mut stat in stats.into_iter() {
+        for mut stat in stats {
             let (send, recv) = channel::bounded(0);
             results.push(recv);
             pool.execute(move || {
@@ -359,7 +362,7 @@ impl Stats {
     #[allow(clippy::option_map_unit_fn)]
     #[inline]
     fn add(&mut self, sample: &[u8]) {
-        let sample_type = FieldType::from_sample(&self.which.dates, sample);
+        let sample_type = FieldType::from_sample(self.which.dates, sample);
         self.typ.merge(sample_type);
 
         let t = self.typ;
@@ -463,7 +466,10 @@ impl Stats {
                 }
             }
         }
-        match self.median.as_mut().and_then(|v| v.median()) {
+        match self.median.as_mut().and_then(|v| match self.typ {
+            TInteger | TFloat => v.median(),
+            _ => None,
+        }) {
             None => {
                 if self.which.median {
                     pieces.push(empty());
@@ -473,7 +479,10 @@ impl Stats {
                 pieces.push(v.to_string());
             }
         }
-        match self.quartiles.as_mut().and_then(|v| v.quartiles()) {
+        match self.quartiles.as_mut().and_then(|v| match self.typ {
+            TInteger | TFloat => v.quartiles(),
+            _ => None,
+        }) {
             None => {
                 if self.which.quartiles {
                     pieces.push(empty());
@@ -514,7 +523,7 @@ impl Stats {
                     pieces.push(
                         v.modes()
                             .iter()
-                            .map(|c| String::from_utf8_lossy(c))
+                            .map(|c| unsafe { String::from_utf8_unchecked(c.to_vec()) })
                             .join(","),
                     );
                 }
@@ -559,7 +568,7 @@ pub enum FieldType {
 
 impl FieldType {
     #[inline]
-    pub fn from_sample(dates: &bool, sample: &[u8]) -> FieldType {
+    pub fn from_sample(dates: bool, sample: &[u8]) -> FieldType {
         if sample.is_empty() {
             return TNull;
         }
@@ -573,7 +582,7 @@ impl FieldType {
         if string.parse::<f64>().is_ok() {
             return TFloat;
         }
-        if *dates {
+        if dates {
             if let Ok(parsed_date) = string.parse::<DateTimeUtc>() {
                 let rfc3339_date_str = parsed_date.0.to_string();
                 let datelen = rfc3339_date_str.len();
@@ -589,12 +598,14 @@ impl FieldType {
         TString
     }
 
-    fn is_number(&self) -> bool {
-        *self == TFloat || *self == TInteger
+    #[inline]
+    fn is_number(self) -> bool {
+        self == TFloat || self == TInteger
     }
 
-    fn is_null(&self) -> bool {
-        *self == TNull
+    #[inline]
+    fn is_null(self) -> bool {
+        self == TNull
     }
 }
 
@@ -747,30 +758,29 @@ impl TypedMinMax {
         self.strings.add(sample.to_vec());
         match typ {
             TString | TUnknown | TNull => {}
-            TFloat => {
-                let n = str::from_utf8(&*sample)
+            TFloat => unsafe {
+                let n = str::from_utf8_unchecked(&*sample)
+                    .parse::<f64>()
                     .ok()
-                    .and_then(|s| s.parse::<f64>().ok())
                     .unwrap();
+
                 self.floats.add(n);
                 self.integers.add(n as i64);
-            }
-            TInteger => {
-                let n = str::from_utf8(&*sample)
+            },
+            TInteger => unsafe {
+                let n = str::from_utf8_unchecked(&*sample)
+                    .parse::<i64>()
                     .ok()
-                    .and_then(|s| s.parse::<i64>().ok())
                     .unwrap();
                 self.integers.add(n);
                 self.floats.add(n as f64);
-            }
-            TDate | TDateTime => {
-                let n = str::from_utf8(&*sample)
-                    .ok()
-                    .and_then(|s| dateparser::parse(s).ok())
-                    .unwrap();
+            },
+            TDate | TDateTime => unsafe {
+                let tempstr = str::from_utf8_unchecked(&*sample);
+                let n = dateparser::parse(tempstr).ok().unwrap();
 
                 self.dates.add(n.to_string());
-            }
+            },
         }
     }
 
@@ -787,11 +797,11 @@ impl TypedMinMax {
         match typ {
             TNull => None,
             TString | TUnknown => match (self.strings.min(), self.strings.max()) {
-                (Some(min), Some(max)) => {
-                    let min = String::from_utf8_lossy(&**min).to_string();
-                    let max = String::from_utf8_lossy(&**max).to_string();
+                (Some(min), Some(max)) => unsafe {
+                    let min = String::from_utf8_unchecked((&**min).to_vec());
+                    let max = String::from_utf8_unchecked((&**max).to_vec());
                     Some((min, max))
-                }
+                },
                 _ => None,
             },
             TDate | TDateTime => match (self.dates.min(), self.dates.max()) {
@@ -823,5 +833,7 @@ impl Commute for TypedMinMax {
 
 #[inline]
 fn from_bytes<T: FromStr>(bytes: &[u8]) -> Option<T> {
-    str::from_utf8(bytes).ok().and_then(|s| s.parse().ok())
+    // we don't need to do UTF-8 validation as we automatically
+    // transcode to UTF-8 the input
+    unsafe { str::from_utf8_unchecked(bytes).parse().ok() }
 }
