@@ -1,32 +1,31 @@
 #![allow(dead_code)]
 use std::borrow::Cow;
-use std::cmp;
-use std::env;
-use std::fs;
-use std::io;
 use std::path::{Path, PathBuf};
-use std::str;
-use std::thread;
-use std::time;
-
-use ::num_cpus;
-use docopt::Docopt;
-use log::{info, log_enabled, Level};
-use serde::de::{Deserialize, DeserializeOwned, Deserializer, Error};
+use std::{cmp, env, fs, io, str, thread, time};
 
 use crate::config::{Config, Delimiter};
 use crate::CliResult;
+use docopt::Docopt;
 use indicatif::{ProgressBar, ProgressStyle};
+use log::{debug, info, log_enabled, Level};
+use regex::Regex;
+use serde::de::{Deserialize, DeserializeOwned, Deserializer, Error};
 use thousands::Separable;
 
 pub fn num_cpus() -> usize {
-    num_cpus::get()
+    thread::available_parallelism().unwrap().get()
 }
 
 const MAX_JOBS_CPU_DIVISOR: usize = 3;
+pub static DEFAULT_USER_AGENT: &str = concat!(
+    env!("CARGO_PKG_NAME"),
+    "/",
+    env!("CARGO_PKG_VERSION"),
+    " (https://github.com/jqnatividad/qsv)",
+);
 
 pub fn max_jobs() -> usize {
-    let cpus = num_cpus::get();
+    let cpus = num_cpus();
     let max_jobs_env = match env::var("QSV_MAX_JOBS") {
         Ok(val) => val.parse::<isize>().unwrap_or_default(),
         Err(_) => 0,
@@ -40,22 +39,21 @@ pub fn max_jobs() -> usize {
 
 pub fn version() -> String {
     let mut enabled_features = "".to_string();
-    if let Some(qsv_type) = option_env!("CARGO_BIN_NAME") {
-        if qsv_type != "qsvlite" {
-            #[cfg(feature = "apply")]
-            enabled_features.push_str("apply;");
-            #[cfg(feature = "foreach")]
-            enabled_features.push_str("foreach;");
-            #[cfg(feature = "generate")]
-            enabled_features.push_str("generate;");
-            #[cfg(feature = "lua")]
-            enabled_features.push_str("lua;");
-            #[cfg(feature = "python")]
-            enabled_features.push_str("python;");
 
-            enabled_features.push('-');
-        }
-    }
+    #[cfg(all(feature = "apply", not(feature = "lite")))]
+    enabled_features.push_str("apply;");
+    #[cfg(all(feature = "fetch", not(feature = "lite")))]
+    enabled_features.push_str("fetch;");
+    #[cfg(all(feature = "foreach", not(feature = "lite")))]
+    enabled_features.push_str("foreach;");
+    #[cfg(all(feature = "generate", not(feature = "lite")))]
+    enabled_features.push_str("generate;");
+    #[cfg(all(feature = "lua", not(feature = "lite")))]
+    enabled_features.push_str("lua;");
+    #[cfg(all(feature = "python", not(feature = "lite")))]
+    enabled_features.push_str("python;");
+
+    enabled_features.push('-');
 
     #[cfg(feature = "mimalloc")]
     let malloc_kind = "mimalloc".to_string();
@@ -133,6 +131,7 @@ pub fn prep_progress(progress: &ProgressBar, record_count: u64) {
         " of {} records",
         record_count.separate_with_commas()
     ));
+    // redraw every 1% of progress
     progress.set_draw_delta(record_count / 100);
 
     if log_enabled!(Level::Info) {
@@ -160,6 +159,7 @@ pub fn finish_progress(progress: &ProgressBar) {
     }
 }
 
+#[cfg(all(any(feature = "apply", feature = "fetch"), not(feature = "lite")))]
 macro_rules! update_cache_info {
     ($progress:expr, $cache_instance:expr) => {
         use cached::Cached;
@@ -174,7 +174,7 @@ macro_rules! update_cache_info {
                     let misses = cache.cache_misses().expect("Cache misses required");
                     let hit_ratio = (hits as f64 / (hits + misses) as f64) * 100.0;
                     $progress.set_message(format!(
-                        " of {} records. Geocode cache hit ratio: {hit_ratio:.2}% - {} entries",
+                        " of {} records. Cache hit ratio: {hit_ratio:.2}% - {} entries",
                         $progress.length().separate_with_commas(),
                         cache_size.separate_with_commas(),
                     ));
@@ -183,8 +183,18 @@ macro_rules! update_cache_info {
             _ => {}
         }
     };
+    ($progress:expr, $cache_hits:expr, $num_rows:expr) => {
+        use thousands::Separable;
+
+        let hit_ratio = ($cache_hits as f64 / $num_rows as f64) * 100.0;
+        $progress.set_message(format!(
+            " of {} records. Redis cache hit ratio: {hit_ratio:.2}%",
+            $progress.length().separate_with_commas(),
+        ));
+    };
 }
 
+#[cfg(all(any(feature = "apply", feature = "fetch"), not(feature = "lite")))]
 pub(crate) use update_cache_info;
 
 pub fn get_args<T>(usage: &str, argv: &[&str]) -> CliResult<T>
@@ -471,3 +481,20 @@ pub fn qsv_check_for_update(bin_name: &str) {
         info!("Up to date ({curr_version})... no update required.");
     };
 }
+
+pub fn safe_header_names(headers: csv::StringRecord, check_first_char: bool) -> Vec<String> {
+    // Create "safe" var/key names
+    // Replace whitespace/invalid chars with _.
+    // If name starts with a number, replace it with an _ as well
+    let re = Regex::new(r"[^A-Za-z0-9]").unwrap();
+    let mut header_vec: Vec<String> = Vec::with_capacity(headers.len());
+    for (_i, h) in headers.iter().take(headers.len()).enumerate() {
+        let mut python_var_name = re.replace_all(h, "_").to_string();
+        if check_first_char && python_var_name.as_bytes()[0].is_ascii_digit() {
+            python_var_name.replace_range(0..1, "_");
+        }
+        header_vec.push(python_var_name);
+    }
+    debug!("safe header names: {header_vec:?}");
+    header_vec
+}
